@@ -106,7 +106,11 @@ skip_if_mocked("libphonenumber doesn't know about location/operators/network-ser
   $number = Number::Phone->new('+442087712924');
   subtest "geo numbers have correct location" => sub {
       plan tests => 2;
+      # call it twice to make sure we cover the situation where N:P:UK::Exchanges
+      # has already been loaded. NB this isn't tickled when we check
+      # with a non-geo number
       my $loc = $number->location();
+      $number->location();
       is($loc->[0], 51.38309,  "latitude");
       is($loc->[1], -0.336079, "longitude");
   };
@@ -242,11 +246,16 @@ foreach my $tuple (
   });
 }
 
-foreach my $invalid (qw(+44275939345 +44208771292 +44113203160 +44113325000)) {
-                  #        Protected    Normal       Normal       Protected
+foreach my $invalid (qw(+442 +44275939345 +44208771292 +44113203160 +44113325000)) {
+                  #   Tiny!   Protected    Normal       Normal       Protected
     $number = Number::Phone->new($invalid);
     ok(!defined($number), "$invalid is invalid (too short)");
 }
+foreach my $invalid (qw(+4427593934500 +4420877129200 +4411320316000 +4411332500000)) {
+                  #        Protected       Normal         Normal         Protected
+    $number = Number::Phone->new($invalid);
+    ok(!defined($number), "$invalid is invalid (too long)");
+}
 
 foreach my $tuple (
   [ 'Number::Phone'     => '+441954202020', '+44 1954 202020' ],
@@ -261,4 +270,44 @@ foreach my $tuple (
   });
 }
 
+note("is_drama");
+my @drama_numbers = (
+        ['Leeds' , '+44113 496 0553', '+44113 496 0494'],
+        ['Sheffield', '+44114 496 0445'],
+        ['Nottingham', '+44115 496 0881'],
+        ['Leicester', '+44116 496 0712'],
+        ['Bristol', '+44117 496 0838'],
+        ['Reading', '+44118 496 0976'],
+        ['Birmingham', '+44121 496 0835'],
+        ['Edinburgh', '+44131 496 0107'],
+        ['Glasgow', '+44141 496 0297'],
+        ['Liverpool', '+44151 496 0787'],
+        ['Mahchester', '+44161 496 0508'],
+        ['London', '+4420 7946 0364', '+4420 7946 0885'],
+        ['Tyneside', '+44191 498 0228'],
+        ['Northern Ireland', '+44289649 6008'],
+        ['Cardiff', '+4429 2018 0678'],
+        ['Mobile', '+44 7700 900011', '+44 7700 900471'],
+        ['Freephone', '+44 8081 570576', '+44 8081 570044'],
+        ['Premium Rate', '+44 909 879 0845'],
+        ['UK-Wide', '+44 3069 990965']
+);
+foreach my $dn (@drama_numbers) {
+     my $area = shift @{$dn};
+     foreach my $num (@{$dn}) {
+          my $phone = Number::Phone->new($num);
+          skip_if_mocked("Stubs don't support is_drama", 1, sub {
+              ok($phone->is_drama(), "$area drama number $num is_drama");
+          });
+     }
+}
+foreach my $number (qw(+447979866975 +442087712924)) {
+     skip_if_mocked("Stubs don't support is_drama", 1, sub {
+         ok(defined(Number::Phone->new($number)->is_drama()) &&
+            !Number::Phone->new($number)->is_drama(),
+            "normal number $number is not is_drama");
+     });
+}
+
+
 1;
